﻿Imports System.Globalization
Imports System.Reflection
Imports Syncfusion.UI.Xaml.Charts
Imports sfChartTest
Imports System.Windows.Threading
Imports System.Collections.ObjectModel

Class MainWindow

    Dim TestColl As ViewModel
    Public BaseTicker As Ticker

    Public XAxis As CustomDateTimeCategoryXAxis
    Public YAxisPrice As CustomLogarithmicYAxis
    Public YAxisAnnotation As NumericalAxis

    Public SeriesPrice As New FastSteplineBitmapSeriesClipExt
    Dim SeriesAnnotation As New FastSteplineBitmapSeriesClipExt

    Dim scrollBar As SfChartResizableBar
    Dim zoomDays As Integer

    Dim BaseBrush As New SolidColorBrush(Color.FromArgb(255, 100, 200, 255))


    Sub New()

        ' This call is required by the designer.
        InitializeComponent()

        ' Add any initialization after the InitializeComponent() call.

        DataContext = Me

        TestColl = New ViewModel
        BaseTicker = TestColl.Tickers(0)

        Axes()
        Series()
        ClearAxes()
        AddRow2()
        AddAnnotationRow()

        'SET NO ANTIALIAS
        Dim image As Image = GetPrivateField(chart, "fastRenderDevice")
        If (Not (image) Is Nothing) Then
            RenderOptions.SetBitmapScalingMode(image, BitmapScalingMode.NearestNeighbor)
        End If

        ZoomX()

    End Sub

    Private Function GetPrivateField(ByVal chart As SfChart, ByVal name As String) As Image
        Dim info As FieldInfo = chart.GetType().GetField(name, BindingFlags.Instance Or (BindingFlags.NonPublic Or BindingFlags.Public))
        If (Not (info) Is Nothing) Then
            Return CType(info.GetValue(chart), Image)
        End If
        Return Nothing
    End Function


    Sub Axes()

        'PRIMARY AXIS
        XAxis = chart.PrimaryAxis
        XAxis.Name = NameOf(XAxis)
        XAxis.LabelFormat = "M/d/yy"
        XAxis.Interval = 1
        XAxis.MainWindow = Me

        'Y AXIS PRICE
        YAxisPrice = New CustomLogarithmicYAxis
        YAxisPrice.Name = NameOf(YAxisPrice)
        YAxisPrice.LogarithmicBase = LogBase
        YAxisPrice.FontSize = 12
        YAxisPrice.ShowGridLines = False
        YAxisPrice.Header = "Price"
        YAxisPrice.MainWindow = Me

        'Y AXIS ANNOTATION
        YAxisAnnotation = New NumericalAxis
        With YAxisAnnotation
            .Name = NameOf(YAxisAnnotation)
            .ShowGridLines = False
            .Visibility = Visibility.Collapsed
        End With

        chart.Header = "Days"

    End Sub

    Sub Series()

        'SERIES PRICE
        With SeriesPrice
            .Name = NameOf(SeriesPrice)
            .XBindingPath = NameOf(Minute.DateTime)
            .YBindingPath = NameOf(Minute.Price)
            .Interior = New SolidColorBrush(Color.FromArgb(255, 0, 255, 0))
            .StrokeThickness = 1
            .YAxis = YAxisPrice
            .ItemsSource = TestColl.Tickers(0).Minutes
        End With
        chart.Series.Add(SeriesPrice)

        'SERIES ANNOTATION
        With SeriesAnnotation
            .YAxis = YAxisAnnotation
        End With
        chart.Series.Add(SeriesAnnotation)

    End Sub

    Sub ClearAxes()

        For i = chart.Axes.Count - 1 To 0 Step -1
            If Not chart.Axes(i) Is XAxis Then
                chart.Axes.RemoveAt(i)
            End If
        Next

    End Sub

    Sub AddRow2()

        chart.RowDefinitions.Add(New ChartRowDefinition With {.BorderThickness = 1, .BorderStroke = New SolidColorBrush(Colors.Red), .Height = 1})
        chart.Axes.Add(YAxisPrice)
        SfChart.SetRow(YAxisPrice, 2)

    End Sub

    Sub AddAnnotationRow()

        SfChart.SetRow(YAxisAnnotation, 0) 'row starts from 0 at the bottom
        SfChart.SetRowSpan(YAxisAnnotation, 3) 'row ends to 3rd row at the top
        chart.Axes.Add(YAxisAnnotation)

    End Sub

    Private Sub btZoomAll_Click(sender As Object, e As RoutedEventArgs) Handles btZoomAll.Click

        zoomDays = 0
        ZoomX()

    End Sub

    Private Sub btZoom1day_Click(sender As Object, e As RoutedEventArgs) Handles btZoom1day.Click

        zoomDays = 1
        ZoomX()

    End Sub

    Sub ZoomX()

        If zoomDays > 0 Then

            XAxis.ZoomPosition = 1 - (zoomDays / BaseTicker.TotalDays)
            XAxis.ZoomFactor = zoomDays / BaseTicker.TotalDays

            XAxis.EnableScrollBar = True
            scrollBar.RangeEnd = 1

            XAxis.IntervalType = DateTimeIntervalType.Days

        Else 'zoom all

            XAxis.ZoomPosition = 0
            XAxis.ZoomFactor = 1
            XAxis.EnableScrollBar = False

            XAxis.IntervalType = DateTimeIntervalType.Years

        End If

    End Sub

    Private Sub chart_Loaded(sender As Object, e As RoutedEventArgs) Handles chart.Loaded
        scrollBar = CType(chart.PrimaryAxis.GetType.GetField("sfChartResizableBar", BindingFlags.NonPublic Or BindingFlags.GetField Or BindingFlags.Instance).GetValue(chart.PrimaryAxis), SfChartResizableBar)
    End Sub

End Class

