﻿Imports System.Reflection
Imports Syncfusion.UI.Xaml.Charts

'Custom date time category axis created for overriding the labels generation method.
Public Class CustomDateTimeCategoryXAxis
    Inherits DateTimeCategoryAxis

    Public MainWindow As MainWindow


    ' Overriding this method can generate the labels As required In accordance With the data.
    Protected Overrides Sub GenerateVisibleLabels()

        If IntervalType = DateTimeIntervalType.Days Then

            Dim area = CType(Me.GetType().GetProperty("Area", BindingFlags.GetProperty Or BindingFlags.NonPublic Or BindingFlags.Instance).GetValue(Me), SfChart)
            area.Annotations.Clear()

            Dim actualSeries As ChartSeriesBase = MainWindow.SeriesPrice
            Dim actualXValues = CType(actualSeries.GetType().GetProperty("ActualXValues", BindingFlags.GetProperty Or BindingFlags.NonPublic Or BindingFlags.Instance).GetValue(actualSeries), System.Collections.IEnumerable)
            Dim XValues = CType(actualXValues, List(Of Double))

            If (Not (XValues) Is Nothing) AndAlso (XValues.Count > 0) Then

                Dim interval As Double = Me.Interval
                Dim position As Integer = VisibleRange.Start - VisibleRange.Start Mod interval
                Dim xStartValue = XValues(position).FromOADate()
                Dim nextDate As DateTime = xStartValue.Date.AddDays(CType(interval, Integer)).Date

                Do While position <= VisibleRange.End

                    Dim xDateTime As DateTime = XValues(position).FromOADate()


                    If xDateTime.Date >= nextDate Then   'ADD NEW DATE LABEL
                        Dim labelString As String = xDateTime.ToString(Me.LabelFormat, Globalization.CultureInfo.CurrentCulture)
                        VisibleLabels.Add(New DateTimeAxisLabel(position, labelString, position))
                        nextDate = xDateTime.AddDays(CType(interval, Integer)).Date

                    End If

                    position += 1
                Loop

            End If
        Else
            MyBase.GenerateVisibleLabels()
        End If

    End Sub


    'We need to update the secondary axis visible range after the primary axis actual range is updated.
    Protected Overrides Sub CalculateVisibleRange(avalableSize As Size)
        MyBase.CalculateVisibleRange(avalableSize)
        Dim area = CType(Me.GetType().GetProperty("Area", BindingFlags.GetProperty Or BindingFlags.NonPublic Or BindingFlags.Instance).GetValue(Me), SfChart)

        For Each axis In area.Axes

            Dim customSecondaryAxis As Object = Nothing

            If TypeOf axis Is CustomLogarithmicYAxis Or TypeOf axis Is CustomNumericalYAxis Then
                customSecondaryAxis = axis
            End If

            If Not customSecondaryAxis Is Nothing Then
                If customSecondaryAxis.IsUpdateNeeded Then

                    customSecondaryAxis.ZoomY(Me.VisibleRange, area)
                    customSecondaryAxis.IsUpdateNeeded = False
                End If

            End If
        Next

    End Sub

End Class

Public Class CustomLogarithmicYAxis
    Inherits LogarithmicAxis

    Public prevXVisibleRange As DoubleRange
    Public prevVisibleRange As DoubleRange
    Public IsUpdateNeeded As Boolean = True
    Public MainWindow As MainWindow


    Protected Overrides Sub CalculateVisibleRange(avalableSize As Size)
        MyBase.CalculateVisibleRange(avalableSize)

        Me.VisibleRange = prevVisibleRange

        Dim area = CType(Me.GetType().GetProperty("Area", BindingFlags.GetProperty Or BindingFlags.NonPublic Or BindingFlags.Instance).GetValue(Me), SfChart)
        Dim primaryAxis = CType(area.PrimaryAxis, ChartAxisBase2D)
        Dim actualXRange = CType(primaryAxis.GetType().GetProperty("ActualRange", BindingFlags.GetProperty Or BindingFlags.NonPublic Or BindingFlags.Instance).GetValue(primaryAxis), DoubleRange)
        If actualXRange.IsEmpty Then Return
        Dim primaryAxisVisibleRange = CalculatePrimaryAxisRange(area, primaryAxis, actualXRange)

        If primaryAxisVisibleRange = prevXVisibleRange Then Return
        IsUpdateNeeded = True

        prevXVisibleRange = primaryAxisVisibleRange

    End Sub

    Private Function CalculatePrimaryAxisRange(area As SfChart, primaryAxis As ChartAxisBase2D, ByRef actualXRange As DoubleRange) As DoubleRange

        Dim rangeStart As Double
        rangeStart = actualXRange.Start + actualXRange.Delta * primaryAxis.ZoomPosition

        Dim rangeEnd As Double
        rangeEnd = rangeStart + primaryAxis.ZoomFactor * actualXRange.Delta

        Dim visibleXRange = New DoubleRange(rangeStart, rangeEnd)

        Return visibleXRange

    End Function

    Public Sub ZoomY(visibleXRange As DoubleRange, area As SfChart)
        Dim MinLogY, MaxLogY As Double
        Dim Ymax, Ymin As Double

        GetExtremes(visibleXRange.Start, visibleXRange.End, Ymax, Ymin, area, Me)

        'Dim visibleXRangeMiddle As Integer = Math.Round(visibleXRange.Start + visibleXRange.Delta / 2)
        'Dim MiddleXRangePrice = MainWindow.BaseTicker.Minutes(visibleXRangeMiddle).Price

        'Dim YpercentRange As Integer = 50
        'Ymax = MiddleXRangePrice * (1 + (YpercentRange / 2) / 100)
        'Ymin = MiddleXRangePrice * (1 - (YpercentRange / 2) / 100)

        MinLogY = Math.Log(Ymin, LogBase)
        MaxLogY = Math.Log(Ymax, LogBase)

        Me.VisibleRange = New DoubleRange(MinLogY, MaxLogY)

        prevVisibleRange = Me.VisibleRange

    End Sub

End Class

Public Class CustomNumericalYAxis
    Inherits NumericalAxis

    Public prevVisibleRange As DoubleRange
    Public IsUpdateNeeded As Boolean = True

    Protected Overrides Sub CalculateVisibleRange(avalableSize As Size)
        MyBase.CalculateVisibleRange(avalableSize)

        Me.VisibleRange = prevVisibleRange

    End Sub


    Public Sub ZoomY(visibleXRange As DoubleRange, area As SfChart)
        Dim Ymax, Ymin As Double

        GetExtremes(visibleXRange.Start, visibleXRange.End, Ymax, Ymin, area, Me)

        Me.VisibleRange = New DoubleRange(0, Ymax)

        prevVisibleRange = Me.VisibleRange

    End Sub

End Class

Public Class FastSteplineBitmapSeriesExt
    Inherits FastStepLineBitmapSeries

    Protected Overrides Sub SetIndividualPoint(index As Integer, obj As Object, replace As Boolean)
        MyBase.SetIndividualPoint(index, obj, replace)
        CType(Me.YAxis, Object).IsUpdateNeeded = True
    End Sub

End Class

Public Class FastSteplineBitmapSeriesClipExt
    Inherits FastStepLineBitmapSeries

    Private customSegment As FastStepLineBitmapSegmentEx
    Public IsCustomSegmentCreated As Boolean

    Protected Overrides Sub SetIndividualPoint(index As Integer, obj As Object, replace As Boolean)
        MyBase.SetIndividualPoint(index, obj, replace)
        CType(Me.YAxis, Object).IsUpdateNeeded = True
    End Sub

    ' Adding custom FastStepLineBitmapSegment.
    Public Overrides Sub CreateSegments()

        If IsCustomSegmentCreated Then Return

        Dim xValues = GetXValues()

        If Segments.Count = 0 Then

            customSegment = New FastStepLineBitmapSegmentEx(ActualXValues, YValues, Me)
            Segments.Add(customSegment)

        ElseIf (ActualXValues IsNot Nothing) Then

            customSegment.SetData(xValues, YValues)

            Dim ActualData = GetType(ChartSeriesBase).GetProperty("ActualData", BindingFlags.GetProperty OrElse BindingFlags.Instance OrElse BindingFlags.NonPublic).GetValue(Me)
            customSegment.Item = ActualData '8 7 7
            '  GetType(FastStepLineBitmapSegment).GetMethod("SetRange", BindingFlags.InvokeMethod OrElse BindingFlags.Instance OrElse BindingFlags.NonPublic).Invoke(customSegment, Nothing) 

            IsCustomSegmentCreated = True

        End If
    End Sub

End Class


Public Class FastStepLineBitmapSegmentEx
    Inherits FastStepLineBitmapSegment

    Private bitmap As WriteableBitmap
    Dim xPixelPoints As List(Of Double)
    Dim yPixelPoints As List(Of Double)
    Dim Color As Color
    Dim actualXAxis As ChartAxisBase2D
    Dim visibleXRangeStart As Integer
    Dim visibleXRangeEnd As Integer
    Dim xChartVals As IList(Of Double)
    Dim yChartVals As IList(Of Double)

    Public Sub New(ByVal series As ChartSeriesBase)
        MyBase.New(series)
    End Sub

    Public Sub New(ByVal xVals As IList(Of Double), ByVal yVals As IList(Of Double), ByVal series As ChartSeriesBase)
        MyBase.New(xVals, yVals, CType(series, AdornmentSeries))

        xPixelPoints = CType(GetType(FastStepLineBitmapSegment).GetField("xValues", BindingFlags.GetField Or BindingFlags.Instance Or BindingFlags.NonPublic).GetValue(Me), List(Of Double)) '144
        yPixelPoints = CType(GetType(FastStepLineBitmapSegment).GetField("yValues", BindingFlags.GetField Or BindingFlags.Instance Or BindingFlags.NonPublic).GetValue(Me), List(Of Double)) '134
        Color = CType(series.Interior, SolidColorBrush).Color
        actualXAxis = CType(series.GetType().GetProperty("ActualXAxis", BindingFlags.GetProperty Or BindingFlags.NonPublic Or BindingFlags.Instance).GetValue(series), ChartAxisBase2D)

        xChartVals = xVals
        yChartVals = yVals
    End Sub

    Public Overrides Sub Update(transformer As IChartTransformer)

        visibleXRangeStart = CInt(Math.Ceiling(actualXAxis.VisibleRange.Start))
        visibleXRangeEnd = CInt(Math.Floor(actualXAxis.VisibleRange.End))

        ' Getting the bitmap surface.
        bitmap = CType(GetType(SfChart).GetMethod("GetFastRenderSurface", BindingFlags.InvokeMethod Or BindingFlags.Instance Or BindingFlags.NonPublic).Invoke(CType(Series, ChartSeries).Area, Nothing), WriteableBitmap) '13

        If transformer IsNot Nothing And Series.DataCount > 0 Then

            Dim chartTransformer = CType(transformer, ChartTransform.ChartCartesianTransformer) '3

            xPixelPoints.Clear()
            yPixelPoints.Clear()

            Dim sw As New Stopwatch
            sw = Stopwatch.StartNew

            ' Converting the chart values to pixel points (xPixelPoints and yPixelPoints lists are updated).
            'GetType(FastStepLineBitmapSegment).GetMethod("CalculatePoints", BindingFlags.InvokeMethod Or BindingFlags.Instance Or BindingFlags.NonPublic).Invoke(Me, New Object() {chartTransformer}) 'this method is too slow for big data!

            CalculatePoints(chartTransformer)

            Debug.WriteLine("CalculatePoints method completed in {0} milliseconds.", sw.ElapsedMilliseconds)

            UpdateVisual()

        End If

    End Sub

    Private Sub CalculatePoints(ByVal cartesianTransformer As ChartTransform.ChartCartesianTransformer)

        Dim xAxis As ChartAxis = cartesianTransformer.XAxis
        Dim cnt As Integer = xChartVals.Count - 1
        Dim start As Integer = 0, endV As Integer = 0

        start = Math.Floor(xAxis.VisibleRange.Start)
        endV = Math.Ceiling(xAxis.VisibleRange.End)

        If start < 0 Then
            start = 0
        End If

        If endV > yChartVals.Count - 1 Then
            endV = yChartVals.Count - 1
        End If

        For i As Integer = start To [endV]
            GetType(FastStepLineBitmapSegment).GetMethod("AddDataPoint", BindingFlags.InvokeMethod Or BindingFlags.Instance Or BindingFlags.NonPublic).
                Invoke(Me, New Object() {cartesianTransformer, i})

        Next

    End Sub

    ' Updating the visual for the segments.
    Private Sub UpdateVisual()
        Dim xStart As Integer
        Dim yStart As Integer
        Dim xEnd As Integer
        Dim yEnd As Integer

        If bitmap Is Nothing Or xPixelPoints.Count = 0 Then Return

        Dim top As Integer = CInt(Series.Clip.Bounds.Top)
        Dim bottom As Integer = CInt(Series.Clip.Bounds.Bottom)

        bitmap.BeginWrite()

        For index = 1 To yPixelPoints.Count - 1

            If Not Double.IsNaN(yPixelPoints(index - 1)) AndAlso Not Double.IsNaN(yPixelPoints(index)) Then 'check if ypixel is not nan

                xStart = CInt(xPixelPoints(index - 1))
                xEnd = CInt(xPixelPoints(index))
                yStart = CInt(yPixelPoints(index - 1))
                yEnd = CInt(yPixelPoints(index))


                If yStart <= bottom AndAlso yStart >= top Then  'YSTART IS INSIDE BOUNDS

                    If yEnd > bottom Then   'YEND IS TOO LOW

                        bitmap.DrawLine(xStart, yStart, xEnd, yStart, Color) 'Rendering of the horizontal stepline
                        bitmap.DrawLine(xEnd, yStart, xEnd, bottom, Color) 'Rendering of the vertical stepline

                    ElseIf yEnd < top Then   'YEND IS TOO HIGH

                        bitmap.DrawLine(xStart, yStart, xEnd, yStart, Color) 'Rendering of the horizontal stepline
                        bitmap.DrawLine(xEnd, yStart, xEnd, top, Color) 'Rendering of the vertical stepline

                    Else 'YEND IS INSIDE BOUNDS 

                        bitmap.DrawLine(xStart, yStart, xEnd, yStart, Color) 'Rendering of the horizontal stepline
                        bitmap.DrawLine(xEnd, yStart, xEnd, yEnd, Color) 'Rendering of the vertical stepline

                    End If

                ElseIf yStart > bottom Then 'YSTART IS TOO LOW

                    If yEnd < bottom AndAlso yEnd > top Then 'YEND IS INSIDE BOUNDS

                        bitmap.DrawLine(xEnd, bottom, xEnd, yEnd, Color) 'Rendering of the vertical stepline

                    ElseIf yEnd < top Then 'YEND IS TOO HIGH

                        bitmap.DrawLine(xEnd, bottom, xEnd, top, Color) 'Rendering of the vertical stepline

                    End If

                ElseIf yStart < top Then 'YSTART IS TOO HIGH

                    If yEnd < bottom AndAlso yEnd > top Then 'YEND IS INSIDE BOUNDS

                        bitmap.DrawLine(xEnd, top, xEnd, yEnd, Color) 'Rendering of the vertical stepline

                    ElseIf yEnd > bottom Then 'YEND IS TOO LOW

                        bitmap.DrawLine(xEnd, top, xEnd, bottom, Color) 'Rendering of the vertical stepline

                    End If

                End If
            End If
        Next

        bitmap.EndWrite()

        GetType(SfChart).GetProperty("CanRenderToBuffer", BindingFlags.GetProperty Or BindingFlags.Instance Or BindingFlags.NonPublic).SetValue(CType(Series, ChartSeries).Area, True)

    End Sub

    Private Sub CreateThumbnail(ByVal filename As String, ByVal image5 As BitmapSource)
        If image5 Is Nothing Then Return

        If filename <> String.Empty Then

            Using stream5 As IO.FileStream = New IO.FileStream(filename, IO.FileMode.Create)
                Dim encoder5 As PngBitmapEncoder = New PngBitmapEncoder()
                encoder5.Frames.Add(BitmapFrame.Create(image5))
                encoder5.Save(stream5)
            End Using
        End If
    End Sub
End Class

Class AreaSeriesExt
        Inherits AreaSeries
        Protected Overrides Sub SetIndividualPoint(index As Integer, obj As Object, replace As Boolean)
            MyBase.SetIndividualPoint(index, obj, replace)
            CType(Me.YAxis, Object).IsUpdateNeeded = True
        End Sub
    End Class

    Class FastRangeAreaBitmapSeriesExt
        Inherits FastRangeAreaBitmapSeries
        Protected Overrides Sub SetIndividualPoint(index As Integer, obj As Object, replace As Boolean)
            MyBase.SetIndividualPoint(index, obj, replace)
            CType(Me.YAxis, Object).IsUpdateNeeded = True
        End Sub
    End Class

